from datetime import timedelta

from django.core.exceptions import ValidationError
from django.http import Http404
from django.shortcuts import get_object_or_404 as _get_object_or_404
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from ninja import Schema
from pydantic import EmailStr, root_validator

from . import models

__all__ = [
    "EmailSerializer",
    "PasswordTokenSerializer",
    "ResetTokenSerializer",
]


class EmailSerializer(Schema):
    email: EmailStr


class PasswordValidateMixin(Schema):
    @root_validator
    def password_validate(cls, values):
        token = values.get("token")

        # get token validation time
        password_reset_token_validation_time = (
            models.get_password_reset_token_expiry_time()
        )

        # find token
        try:
            reset_password_token = _get_object_or_404(
                models.ResetPasswordToken, key=token
            )
        except (
            TypeError,
            ValueError,
            ValidationError,
            Http404,
            models.ResetPasswordToken.DoesNotExist,
        ):
            raise Http404(
                _("The OTP password entered is not valid. Please check and try again.")
            )

        # check expiry date
        expiry_date = reset_password_token.created_at + timedelta(
            hours=password_reset_token_validation_time
        )

        if timezone.now() > expiry_date:
            # delete expired token
            reset_password_token.delete()
            raise Http404(_("The token has expired"))
        return values


class PasswordTokenSerializer(PasswordValidateMixin):
    password: str
    token: str


class ResetTokenSerializer(PasswordValidateMixin):
    token: str



import django


def pytest_configure(config):
    from django.conf import settings

    MIDDLEWARE = (
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    )

    settings.configure(
        DEBUG_PROPAGATE_EXCEPTIONS=True,
        DATABASES={
            "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
            "other": {"ENGINE": "django.db.backends.sqlite3", "NAME": "other"},
        },
        SITE_ID=1,
        SECRET_KEY="-ca82+nb!ay6-gq=6t_cps(*9tlp+t#sd%trg3avne!8v4x9#b",
        USE_I18N=True,
        USE_L10N=True,
        STATIC_URL="/static/",
        ROOT_URLCONF="tests.urls",
        TEMPLATES=[
            {
                "BACKEND": "django.template.backends.django.DjangoTemplates",
                "APP_DIRS": True,
            },
        ],
        MIDDLEWARE=MIDDLEWARE,
        MIDDLEWARE_CLASSES=MIDDLEWARE,
        INSTALLED_APPS=(
            "django.contrib.admin",
            "django.contrib.auth",
            "django.contrib.contenttypes",
            "django.contrib.sessions",
            "django.contrib.messages",
            "django.contrib.staticfiles",
            "ninja_extra",
            # include multi token auth
            "django_rest_passwordreset",
            "tests",
        ),
        PASSWORD_HASHERS=("django.contrib.auth.hashers.MD5PasswordHasher",),
        AUTH_PASSWORD_VALIDATORS=[
            {
                "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
            },
            {
                "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
            },
            {
                "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
            },
            {
                "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
            },
        ],
    )

    django.setup()



from django.contrib.auth.models import User

try:
    from unittest.mock import patch
except:
    # Python 2.7 fallback
    from mock import patch

# try getting reverse from django.urls
try:
    # Django 1.10 +
    from django.urls import reverse
except:
    # Django 1.8 and 1.9
    from django.core.urlresolvers import reverse


__all__ = ["HelperMixin", "patch"]


class HelperMixin:
    """
    Mixin which encapsulates methods for login, logout, request reset password and reset password confirm
    """

    def setUpUrls(self):
        """set up urls by using djangos reverse function"""
        self.reset_password_request_url = reverse(
            "password_reset:reset-password-request"
        )
        self.reset_password_confirm_url = reverse(
            "password_reset:reset-password-confirm"
        )
        self.reset_password_validate_token_url = reverse(
            "password_reset:reset-password-validate"
        )

    def django_check_login(self, username, password):
        """
        Checks the django login by querying the user from the database and calling check_password()
        :param username:
        :param password:
        :return:
        """
        user = User.objects.filter(username=username).first()

        return user.check_password(password)

    def rest_do_request_reset_token(
        self, email, HTTP_USER_AGENT="", REMOTE_ADDR="127.0.0.1"
    ):
        """REST API wrapper for requesting a password reset token"""
        data = {"email": email}

        return self.client.post(
            self.reset_password_request_url,
            data,
            format="json",
            HTTP_USER_AGENT=HTTP_USER_AGENT,
            REMOTE_ADDR=REMOTE_ADDR,
            content_type="application/json",
        )

    def rest_do_reset_password_with_token(
        self, token, new_password, HTTP_USER_AGENT="", REMOTE_ADDR="127.0.0.1"
    ):
        """REST API wrapper for resetting a user's password using a token"""
        data = {"token": token, "password": new_password}

        return self.client.post(
            self.reset_password_confirm_url,
            data,
            format="json",
            HTTP_USER_AGENT=HTTP_USER_AGENT,
            REMOTE_ADDR=REMOTE_ADDR,
            content_type="application/json",
        )

    def rest_do_validate_token(
        self, token, HTTP_USER_AGENT="", REMOTE_ADDR="127.0.0.1"
    ):
        """REST API wrapper for validating a token"""
        data = {"token": token}

        return self.client.post(
            self.reset_password_validate_token_url,
            data,
            format="json",
            HTTP_USER_AGENT=HTTP_USER_AGENT,
            REMOTE_ADDR=REMOTE_ADDR,
            content_type="application/json",
        )



""" An extension of django ninja framework, providing a configurable password reset strategy"""
__version__ = "1.2.4"

default_app_config = "django_rest_passwordreset.apps.DjangoRestPasswordResetConfig"



""" contains basic admin views for MultiToken """
from django.contrib import admin

from .models import ResetPasswordToken


@admin.register(ResetPasswordToken)
class ResetPasswordTokenAdmin(admin.ModelAdmin):
    list_display = ("user", "key", "created_at", "ip_address", "user_agent")



import datetime

from django.core.management.base import BaseCommand
from django.utils import timezone

from ...models import clear_expired, get_password_reset_token_expiry_time


class Command(BaseCommand):
    help = "Can be run as a cronjob or directly to clean out expired tokens"

    def handle(self, *args, **options):
        # datetime.now minus expiry hours
        now_minus_expiry_time = timezone.now() - datetime.timedelta(
            hours=get_password_reset_token_expiry_time()
        )
        clear_expired(now_minus_expiry_time)



from ninja_extra import NinjaExtraAPI

from django_rest_passwordreset.controller import ResetPasswordController

api = NinjaExtraAPI(urls_namespace="password_reset")
api.register_controllers(ResetPasswordController)
